/*======================================================================*/
/*----------------------------------------------------------------------*/
/* line.c - Line Scan Conversion					*/
/* 	    An XTango animation of Bresenham's Midpoint Line Algorithm	*/
/* 	    by Anthony G. Francis, Jr.					*/
/*----------------------------------------------------------------------*/
/* References:	_Computer Graphics, Principles and Practice_, 2e,	*/
/*		Foley, et. al.						*/
/*----------------------------------------------------------------------*/
/* Algorithm File 							*/
/*----------------------------------------------------------------------*/
/*======================================================================*/

/*======================================================================*/
/* #include Files							*/
/*======================================================================*/
#include <stdio.h>
#include <math.h>
#include "xtango.h"

/*======================================================================*/
/* #define Statements							*/
/*======================================================================*/

/*======================================================================*/
/* XTango Declarations							*/
/*======================================================================*/
void
  ANIMInit(),
  ANIMVertex(),
  ANIMDrawLine(),
  ANIMDrawPoint(),
  ANIMGoEast(),
  ANIMGoNorthEast(),
  ANIMChangeDecision(),
  ANIMCleanup();


/*======================================================================*/
/* Main Function							*/
/*======================================================================*/
main()
{

/*-Data Structures------------------------------------------------------*/
  int	
    iEndpointX, iEndpointY,		/* Destination Points		*/
    iX, iY,				/* Stepping Variable		*/
    iDeltaX, iDeltaY,			/* Delta X and Delta Y		*/
    iDecide,				/* Decision Variable 		*/
    iIncrementE, iIncrementNE,		/* Increment Values		*/
    i,j,k;				/* Loop Variables		*/

  char *cpReply; 			/* Ptr for XTango return vals	*/

  /* Set up the XTango System */
  TANGOalgoOp("BEGIN");
  ANIMInit();

  /* Brag to the User */
  printf("Bresenham's Midpoint Line Scan-conversion Algorithm\n");
  printf("Adapted for XTango by Anthony G. Francis, Jr.\n");
 
  /*-Input the X and Y coordinates for the algorithm=-------------------*/
  printf ("Select the destination vertex with the mouse.\n");
  printf ("Please select a destination in the first octant.\n");

  iEndpointX = 0;
  iEndpointY = 0;

  /* Note we must stay in the first octant */
  while ((iEndpointX==0 && iEndpointY==0) || iEndpointX>20 ||
	   iEndpointY>iEndpointX) {
    /* Get user input from XTango */
    ANIMVertex(&iEndpointX,&iEndpointY);
  }
  
  
  /*--------------------------------------------------------------------*/
  /* Single Source Shortest Paths Algorithm				*/
  /*--------------------------------------------------------------------*/
  /*-Initialize Data Structures-----------------------------------------*/
  iDeltaX 	= iEndpointX;
  iDeltaY 	= iEndpointY;
  iDecide 	= 2 *  iDeltaY - iDeltaX;
  iIncrementE 	= 2 *  iDeltaY;
  iIncrementNE 	= 2 * (iDeltaY - iDeltaX);
  iX 		= 0;
  iY 		= 0;

  /* Draw the Target Line */
  printf ("Drawing the ideal line and setting up for scan conversion.\n");
  ANIMDrawLine(iEndpointX,iEndpointY);

  /* Draw the First Point */
  printf ("Beginning scan conversion;  Drawing first pixel at (%d, %d)\n",
	  iX,iY);
  ANIMDrawPoint(0,0);

   /* Loop through all the points on the line */
   while (iX < iEndpointX) {
     if (iDecide <= 0) {
       /* Choose East Direction */
       printf ("East direction chosen;      ");
       ANIMGoEast();

       /* Update Point */
       iX++;

       /* Draw Pixel at the Point */
       printf ("Drawing pixel at (%d, %d).\n",iX,iY);
       ANIMDrawPoint(iX,iY);

       /* Update Decision Variable */
       printf ("Updating the decision variable from %d to %d.\n",
	       iDecide, iDecide + iIncrementE);
       ANIMChangeDecision(iDecide, iDecide + iIncrementE, 0);
       iDecide += iIncrementE;

     } else {
       /* Choose Northeast Direction */
       printf ("Northeast direction chosen; ");
       ANIMGoNorthEast();

       /* Update Point */
       iX++;
       iY++;

       /* Draw Pixel at the Point */
       printf ("Drawing pixel at (%d, %d).\n",iX,iY);
       ANIMDrawPoint(iX,iY);

       /* Update Decision Variable */
       printf ("Updating the decision variable from %d to %d.\n",
	       iDecide, iDecide + iIncrementNE);
       ANIMChangeDecision(iDecide, iDecide + iIncrementNE, 1);
       iDecide += iIncrementNE;

     }

   }

   printf("The Line is Complete.\n");
   ANIMCleanup();

   /*-End the Program---------------------------------------------------*/
   TANGOalgoOp("END");

}
